@@ -111,7 +111,7 @@ bp_video_bus_element_sync_message (GstBus *bus, GstMessage *message, BansheePlay
     g_mutex_unlock (player->mutex);
 
     if (found_xoverlay) {
-        gst_x_overlay_set_xwindow_id (player->xoverlay, GDK_WINDOW_XWINDOW (player->video_window));
+        gst_x_overlay_set_xwindow_id (player->xoverlay, player->video_window_xid);
     }
 
     #endif
@@ -126,14 +126,26 @@ _bp_video_pipeline_setup (BansheePlayer *player, GstBus *bus)
 {
     GstElement *videosink;
     
-    #ifdef GDK_WINDOWING_X11
-    
     g_return_if_fail (IS_BANSHEE_PLAYER (player));
     
+     if (player->video_pipeline_setup_cb != NULL) {
+        videosink = player->video_pipeline_setup_cb (player, bus);
+        if (videosink != NULL && GST_IS_ELEMENT (videosink)) {
+            g_object_set (G_OBJECT (player->playbin), "video-sink", videosink, NULL);
+            player->video_display_context_type = BP_VIDEO_DISPLAY_CONTEXT_CUSTOM;
+            return;
+        }
+    }
+    
+    #ifdef GDK_WINDOWING_X11
+
+    player->video_display_context_type = BP_VIDEO_DISPLAY_CONTEXT_GDK_WINDOW;
+    
     videosink = gst_element_factory_make ("gconfvideosink", "videosink");
     if (videosink == NULL) {
         videosink = gst_element_factory_make ("ximagesink", "videosink");
         if (videosink == NULL) {
+            player->video_display_context_type = BP_VIDEO_DISPLAY_CONTEXT_UNSUPPORTED;
             videosink = gst_element_factory_make ("fakesink", "videosink");
             if (videosink != NULL) {
                 g_object_set (G_OBJECT (videosink), "sync", TRUE, NULL);
@@ -150,7 +162,38 @@ _bp_video_pipeline_setup (BansheePlayer *player, GstBus *bus)
         g_signal_connect (videosink, "element-added", G_CALLBACK (bp_video_sink_element_added), player);
     }
     
+    #else
+    
+    player->video_display_context_type = BP_VIDEO_DISPLAY_CONTEXT_UNSUPPORTED;
+
+    #ifndef WIN32
+
+    videosink = gst_element_factory_make ("fakesink", "videosink");
+    if (videosink != NULL) {
+        g_object_set (G_OBJECT (videosink), "sync", TRUE, NULL);
+    }
+    
+    g_object_set (G_OBJECT (player->playbin), "video-sink", videosink, NULL);
+
     #endif
+    
+    #endif
+
+    if (player->video_prepare_window_cb != NULL) {
+        player->video_prepare_window_cb (player);
+    }
+}
+
+P_INVOKE void
+bp_set_video_pipeline_setup_callback (BansheePlayer *player, BansheePlayerVideoPipelineSetupCallback cb)
+{
+    SET_CALLBACK (video_pipeline_setup_cb);
+}
+
+P_INVOKE void
+bp_set_video_prepare_window_callback (BansheePlayer *player, BansheePlayerVideoPrepareWindowCallback cb)
+{
+    SET_CALLBACK (video_prepare_window_cb);
 }
 
 // ---------------------------------------------------------------------------
@@ -159,25 +202,37 @@ _bp_video_pipeline_setup (BansheePlayer *player, GstBus *bus)
 
 #ifdef GDK_WINDOWING_X11
 
-P_INVOKE gboolean
-bp_video_is_supported (BansheePlayer *player)
+P_INVOKE BpVideoDisplayContextType
+bp_video_get_display_context_type (BansheePlayer *player)
 {
-    return TRUE; // bp_video_find_xoverlay (player);
+    return player->video_display_context_type;
 }
 
 P_INVOKE void
-bp_video_set_window (BansheePlayer *player, GdkWindow *window)
+bp_video_set_display_context (BansheePlayer *player, gpointer context)
 {
     g_return_if_fail (IS_BANSHEE_PLAYER (player));
     
-    player->video_window = window;
+    if (bp_video_get_display_context_type (player) == BP_VIDEO_DISPLAY_CONTEXT_GDK_WINDOW) {
+        player->video_window = (GdkWindow *)context;
+    }
+}
+
+P_INVOKE gpointer
+bp_video_get_display_context (BansheePlayer *player)
+{
+    g_return_val_if_fail (IS_BANSHEE_PLAYER (player), NULL);
+   
+    if (bp_video_get_display_context_type (player) == BP_VIDEO_DISPLAY_CONTEXT_GDK_WINDOW) {
+        return player->video_window;
+    }
+    
+    return NULL;
 }
 
 P_INVOKE void
 bp_video_window_expose (BansheePlayer *player, GdkWindow *window, gboolean direct)
 {
-    XID window_id;
-    
     g_return_if_fail (IS_BANSHEE_PLAYER (player));
     
     if (direct && player->xoverlay != NULL && GST_IS_X_OVERLAY (player->xoverlay)) {
@@ -195,27 +250,51 @@ bp_video_window_expose (BansheePlayer *player, GdkWindow *window, gboolean direc
     gst_object_ref (player->xoverlay);
     g_mutex_unlock (player->mutex);
 
-    window_id = GDK_WINDOW_XWINDOW (window);
-
-    gst_x_overlay_set_xwindow_id (player->xoverlay, window_id);
+    gst_x_overlay_set_xwindow_id (player->xoverlay, player->video_window_xid);
     gst_x_overlay_expose (player->xoverlay);
 
     gst_object_unref (player->xoverlay);
 }
 
+// MUST be called from the GTK main thread; calling it in OnRealized will do the right thing.
+P_INVOKE void
+bp_video_window_realize (BansheePlayer *player, GdkWindow *window)
+{
+    g_return_if_fail (IS_BANSHEE_PLAYER (player));
+
+// Code commented out - this requires including gtk/gtk.h for GTK_CHECK_VERSION, which requires too many
+// buildsystem changes for the benefit of a single debug message in the failure case.
+//
+//#if GTK_CHECK_VERSION(2,18,0)
+//    //Explicitly create the native window.  GDK_WINDOW_XWINDOW will call this
+//    //function anyway, but this way we can raise a more useful message should it fail.
+//    if (!gdk_window_ensure_native (window)) {
+//        banshee_log (BANSHEE_LOG_TYPE_ERROR, "player-video", "Couldn't create native window needed for GstXOverlay!");
+//    }
+//#endif
+
+    player->video_window_xid = GDK_WINDOW_XID (window);
+}
+
 #else /* GDK_WINDOWING_X11 */
 
-P_INVOKE gboolean
-bp_video_is_supported (BansheePlayer *player)
+P_INVOKE BpVideoDisplayContextType
+bp_video_get_display_context_type (BansheePlayer *player)
 {
-    return FALSE;
+    return player->video_display_context_type;
 }
 
 P_INVOKE void
-bp_video_set_window (BansheePlayer *player, GdkWindow *window)
+bp_video_set_display_context (BansheePlayer *player, gpointer context)
 {
 }
 
+P_INVOKE gpointer
+bp_video_get_display_context (BansheePlayer *player)
+{
+    return NULL;
+}
+
 P_INVOKE void
 bp_video_window_expose (BansheePlayer *player, GdkWindow *window, gboolean direct)
 {
