@@ -61,6 +61,7 @@ namespace Banshee.GStreamer
     internal delegate void BansheePlayerNextTrackStartingCallback (IntPtr player);
     internal delegate void BansheePlayerAboutToFinishCallback (IntPtr player);
     internal delegate IntPtr VideoPipelineSetupHandler (IntPtr player, IntPtr bus);
+    internal delegate void VideoPrepareWindowHandler (IntPtr player);
     internal delegate void BansheePlayerVolumeChangedCallback (IntPtr player, double newVolume);
 
     internal delegate void GstTaggerTagFoundCallback (IntPtr player, string tagName, ref GLib.Value value);
@@ -85,6 +86,7 @@ namespace Banshee.GStreamer
         private BansheePlayerBufferingCallback buffering_callback;
         private BansheePlayerVisDataCallback vis_data_callback;
         private VideoPipelineSetupHandler video_pipeline_setup_callback;
+        private VideoPrepareWindowHandler video_prepare_window_callback;
         private GstTaggerTagFoundCallback tag_found_callback;
         private BansheePlayerNextTrackStartingCallback next_track_starting_callback;
         private BansheePlayerAboutToFinishCallback about_to_finish_callback;
@@ -149,6 +151,7 @@ namespace Banshee.GStreamer
             buffering_callback = new BansheePlayerBufferingCallback (OnBuffering);
             vis_data_callback = new BansheePlayerVisDataCallback (OnVisualizationData);
             video_pipeline_setup_callback = new VideoPipelineSetupHandler (OnVideoPipelineSetup);
+            video_prepare_window_callback = new VideoPrepareWindowHandler (OnVideoPrepareWindow);
             tag_found_callback = new GstTaggerTagFoundCallback (OnTagFound);
             next_track_starting_callback = new BansheePlayerNextTrackStartingCallback (OnNextTrackStarting);
             about_to_finish_callback = new BansheePlayerAboutToFinishCallback (OnAboutToFinish);
@@ -163,6 +166,7 @@ namespace Banshee.GStreamer
             bp_set_tag_found_callback (handle, tag_found_callback);
             bp_set_next_track_starting_callback (handle, next_track_starting_callback);
             bp_set_video_pipeline_setup_callback (handle, video_pipeline_setup_callback);
+            bp_set_video_prepare_window_callback (handle, video_prepare_window_callback);
             bp_set_volume_changed_callback (handle, volume_changed_callback);
 
             next_track_set = new EventWaitHandle (false, EventResetMode.ManualReset);
@@ -304,6 +308,9 @@ namespace Banshee.GStreamer
         private void OnNextTrackStarting (IntPtr player)
         {
             if (GaplessEnabled) {
+                // Must do it here because the next track is already playing.
+                ServiceManager.PlayerEngine.IncrementLastPlayed (1.0);
+
                 OnEventChanged (PlayerEvent.EndOfStream);
                 OnEventChanged (PlayerEvent.StartOfStream);
             }
@@ -683,6 +690,11 @@ namespace Banshee.GStreamer
             return clutter_video_sink;
         }
 
+        private void OnVideoPrepareWindow (IntPtr player)
+        {
+            OnEventChanged (PlayerEvent.PrepareVideoWindow);
+        }
+
 #endregion
 
 #region Preferences
@@ -781,6 +793,10 @@ namespace Banshee.GStreamer
         private static extern void bp_set_tag_found_callback (HandleRef player,
             GstTaggerTagFoundCallback cb);
 
+        [DllImport ("libbanshee.dll")]
+        private static extern void bp_set_video_prepare_window_callback (HandleRef player,
+           VideoPrepareWindowHandler cb);
+
         [DllImport ("libbanshee.dll")]
         private static extern void bp_set_next_track_starting_callback (HandleRef player,
             BansheePlayerNextTrackStartingCallback cb);
