@@ -97,7 +97,11 @@ namespace Booter
             } else if (ApplicationContext.CommandLine.Contains ("client")) {
                 BootClient (Path.GetFileNameWithoutExtension (ApplicationContext.CommandLine["client"]));
             } else {
-                BootClient ("Nereid");
+                if (PlatformDetection.IsMeeGo) {
+                    BootClient ("MeeGo");
+                } else {
+                    BootClient ("Nereid");
+                }
             }
         }
 
@@ -157,6 +161,7 @@ namespace Booter
                 new LayoutGroup ("playback", Catalog.GetString ("Playback Control Options"),
                     new LayoutOption ("next", Catalog.GetString ("Play the next track, optionally restarting if the 'restart' value is set")),
                     new LayoutOption ("previous", Catalog.GetString ("Play the previous track, optionally restarting if the 'restart value is set")),
+                    new LayoutOption ("restart-or-previous", Catalog.GetString ("If the current song has been played longer than 4 seconds then restart it, otherwise the same as --previous")),
                     new LayoutOption ("play-enqueued", Catalog.GetString ("Automatically start playing any tracks enqueued on the command line")),
                     new LayoutOption ("play", Catalog.GetString ("Start playback")),
                     new LayoutOption ("pause", Catalog.GetString ("Pause playback")),
@@ -164,8 +169,9 @@ namespace Booter
                     new LayoutOption ("stop", Catalog.GetString ("Completely stop playback")),
                     new LayoutOption ("stop-when-finished", Catalog.GetString (
                         "Enable or disable playback stopping after the currently playing track (value should be either 'true' or 'false')")),
-                    new LayoutOption ("set-volume=LEVEL", Catalog.GetString ("Set the playback volume (0-100)")),
-                    new LayoutOption ("set-position=POS", Catalog.GetString ("Seek to a specific point (seconds, float)"))
+                    new LayoutOption ("set-volume=LEVEL", Catalog.GetString ("Set the playback volume (0-100), prefix with +/- for relative values")),
+                    new LayoutOption ("set-position=POS", Catalog.GetString ("Seek to a specific point (seconds, float)")),
+                    new LayoutOption ("set-rating=RATING", Catalog.GetString ("Set the currently played track's rating (0 to 5)"))
                 ),
 
                 new LayoutGroup ("query-player", Catalog.GetString ("Player Engine Query Options"),
@@ -195,7 +201,11 @@ namespace Booter
                 new LayoutGroup ("ui", Catalog.GetString ("User Interface Options"),
                     new LayoutOption ("show|--present", Catalog.GetString ("Present the user interface on the active workspace")),
                     new LayoutOption ("hide", Catalog.GetString ("Hide the user interface")),
-                    new LayoutOption ("no-present", Catalog.GetString ("Do not present the user interface, regardless of any other options"))
+                    new LayoutOption ("no-present", Catalog.GetString ("Do not present the user interface, regardless of any other options")),
+                    new LayoutOption ("show-import-media", Catalog.GetString ("Present the import media dialog box")),
+                    new LayoutOption ("show-about", Catalog.GetString ("Present the about dialog")),
+                    new LayoutOption ("show-open-location", Catalog.GetString ("Present the open location dialog")),
+                    new LayoutOption ("show-preferences", Catalog.GetString ("Present the preferences dialog"))
                 ),
 
                 new LayoutGroup ("debugging", Catalog.GetString ("Debugging and Development Options"),
