@@ -31,7 +31,9 @@ using System.Collections.Generic;
 using Mono.Unix;
 using Gtk;
 
-using Banshee.Base;
+using Hyena;
+using Hyena.Widgets;
+
 using Banshee.Query;
 using Banshee.Sources;
 using Banshee.Library;
@@ -51,26 +53,26 @@ namespace Banshee.Gui
 
         private static readonly string [] require_selection_actions = new string [] {
             "TrackContextMenuAction", "TrackPropertiesAction", "AddToPlaylistAction",
-            "RemoveTracksAction", "RemoveTracksFromLibraryAction", "DeleteTracksFromDriveAction",
-            "RateTracksAction", "SelectNoneAction"
+            "RemoveTracksAction", "RemoveTracksFromLibraryAction", "OpenContainingFolderAction",
+            "DeleteTracksFromDriveAction", "RateTracksAction", "SelectNoneAction"
         };
-        
+
         public event EventHandler SelectionChanged;
 
-        public TrackActions (InterfaceActionService actionService) : base (actionService, "Track")
+        public TrackActions () : base ("Track")
         {
             Add (new ActionEntry [] {
-                new ActionEntry("TrackContextMenuAction", null, 
+                new ActionEntry("TrackContextMenuAction", null,
                     String.Empty, null, null, OnTrackContextMenu),
 
                 new ActionEntry("SelectAllAction", null,
                     Catalog.GetString("Select _All"), "<control>A",
                     Catalog.GetString("Select all tracks"), OnSelectAll),
-                    
+
                 new ActionEntry("SelectNoneAction", null,
                     Catalog.GetString("Select _None"), "<control><shift>A",
                     Catalog.GetString("Unselect all tracks"), OnSelectNone),
-                    
+
                 new ActionEntry ("TrackEditorAction", Stock.Edit,
                     Catalog.GetString ("_Edit Track Information"), "E",
                     Catalog.GetString ("Edit information on selected tracks"), OnTrackEditor),
@@ -97,6 +99,10 @@ namespace Banshee.Gui
                     Catalog.GetString ("Remove From _Library"), "",
                     Catalog.GetString ("Remove selected track(s) from library"), OnRemoveTracksFromLibrary),
 
+                new ActionEntry ("OpenContainingFolderAction", null,
+                    Catalog.GetString ("_Open Containing Folder"), null,
+                    Catalog.GetString ("Open the folder that contains the selected item"), OnOpenContainingFolder),
+
                 new ActionEntry ("DeleteTracksFromDriveAction", null,
                     Catalog.GetString ("_Delete From Drive"), null,
                     Catalog.GetString ("Permanently delete selected item(s) from medium"), OnDeleteTracksFromDrive),
@@ -134,14 +140,14 @@ namespace Banshee.Gui
                 current_source.TrackModel.Selection.Changed -= HandleSelectionChanged;
                 current_source = null;
             }
-            
+
             ITrackModelSource new_source = ActiveSource as ITrackModelSource;
             if (new_source != null) {
                 new_source.TrackModel.Selection.Changed += HandleSelectionChanged;
                 current_source = new_source;
             }
-            
-            Banshee.Base.ThreadAssist.ProxyToMain (UpdateActions);
+
+            ThreadAssist.ProxyToMain (UpdateActions);
         }
 
         private void HandleActionsChanged (object sender, EventArgs args)
@@ -156,7 +162,7 @@ namespace Banshee.Gui
 
         private void HandleSelectionChanged (object sender, EventArgs args)
         {
-            Banshee.Base.ThreadAssist.ProxyToMain (delegate {
+            ThreadAssist.ProxyToMain (delegate {
                 OnSelectionChanged ();
                 UpdateActions ();
             });
@@ -166,7 +172,7 @@ namespace Banshee.Gui
         {
             ResetRating ();
         }
-        
+
         private void OnSelectionChanged ()
         {
             EventHandler handler = SelectionChanged;
@@ -203,7 +209,7 @@ namespace Banshee.Gui
             Source source = ServiceManager.SourceManager.ActiveSource;
             bool in_database = source is DatabaseSource;
             PrimarySource primary_source = (source as PrimarySource) ?? (source.Parent as PrimarySource);
-            
+
             Hyena.Collections.Selection selection = (source is ITrackModelSource) ? (source as ITrackModelSource).TrackModel.Selection : null;
 
             if (selection != null) {
@@ -227,16 +233,23 @@ namespace Banshee.Gui
 
                     UpdateAction ("RemoveTracksAction", is_track_source && track_source.CanRemoveTracks, has_selection, source);
                     UpdateAction ("DeleteTracksFromDriveAction", is_track_source && track_source.CanDeleteTracks, has_selection, source);
+
+                    //if it can delete tracks, most likely it can open their folder
+                    UpdateAction ("OpenContainingFolderAction", is_track_source && track_source.CanDeleteTracks, has_single_selection, source);
+
                     UpdateAction ("RemoveTracksFromLibraryAction", source.Parent is LibrarySource, has_selection, null);
-                    
+
                     UpdateAction ("TrackPropertiesAction", source.HasViewableTrackProperties, has_selection, source);
                     UpdateAction ("TrackEditorAction", source.HasEditableTrackProperties, has_selection, source);
                     UpdateAction ("RateTracksAction", in_database, has_selection, null);
                     UpdateAction ("AddToPlaylistAction", in_database && primary_source != null &&
                             primary_source.SupportsPlaylists && !primary_source.PlaylistsReadOnly, has_selection, null);
 
-                    if (primary_source != null) {
-                        this["DeleteTracksFromDriveAction"].Label = String.Format (Catalog.GetString ("_Delete From {0}"), primary_source.StorageName);
+                    if (primary_source != null &&
+                        !(primary_source is LibrarySource) &&
+                        primary_source.StorageName != null) {
+                        this["DeleteTracksFromDriveAction"].Label = String.Format (
+                            Catalog.GetString ("_Delete From \"{0}\""), primary_source.StorageName);
                     }
                 }
             } else {
@@ -248,7 +261,7 @@ namespace Banshee.Gui
         {
             if (current_source != null) {
                 int rating = 0;
-    
+
                 // If there is only one track, get the preset rating
                 if (current_source.TrackModel.Selection.Count == 1) {
                     foreach (TrackInfo track in current_source.TrackModel.SelectedItems) {
@@ -260,7 +273,7 @@ namespace Banshee.Gui
         }
 
 #endregion
-            
+
 #region Action Handlers
 
         private void OnSelectAll (object o, EventArgs args)
@@ -284,15 +297,15 @@ namespace Banshee.Gui
         private bool RunSourceOverrideHandler (string sourceOverrideHandler)
         {
             Source source = current_source as Source;
-            InvokeHandler handler = source != null 
-                ? source.GetInheritedProperty<InvokeHandler> (sourceOverrideHandler) 
+            InvokeHandler handler = source != null
+                ? source.GetInheritedProperty<InvokeHandler> (sourceOverrideHandler)
                 : null;
-            
+
             if (handler != null) {
                 handler ();
                 return true;
             }
-            
+
             return false;
         }
 
@@ -302,7 +315,7 @@ namespace Banshee.Gui
                 Banshee.Gui.TrackEditor.TrackEditorDialog.RunView (current_source.TrackModel);
             }
         }
-        
+
         private void OnTrackEditor (object o, EventArgs args)
         {
             if (current_source != null && !RunSourceOverrideHandler ("TrackEditorActionHandler")) {
@@ -316,6 +329,11 @@ namespace Banshee.Gui
         {
             Source active_source = ServiceManager.SourceManager.ActiveSource;
 
+            List<Source> children;
+            lock (ActivePrimarySource.Children) {
+                children = new List<Source> (ActivePrimarySource.Children);
+            }
+
             // TODO find just the menu that was activated instead of modifying all proxies
             foreach (Widget proxy_widget in (o as Gtk.Action).Proxies) {
                 MenuItem menu = proxy_widget as MenuItem;
@@ -327,15 +345,15 @@ namespace Banshee.Gui
 
                 submenu.Append (this ["AddToNewPlaylistAction"].CreateMenuItem ());
                 bool separator_added = false;
-                
-                foreach (Source child in ActivePrimarySource.Children) {
+
+                foreach (Source child in children) {
                     PlaylistSource playlist = child as PlaylistSource;
                     if (playlist != null) {
                         if (!separator_added) {
                             submenu.Append (new SeparatorMenuItem ());
                             separator_added = true;
                         }
-                        
+
                         PlaylistMenuItem item = new PlaylistMenuItem (playlist);
                         item.Image = new Gtk.Image ("playlist-source", IconSize.Menu);
                         item.Activated += OnAddToExistingPlaylist;
@@ -343,7 +361,7 @@ namespace Banshee.Gui
                         submenu.Append (item);
                     }
                 }
-                
+
                 submenu.ShowAll ();
             }
         }
@@ -351,7 +369,7 @@ namespace Banshee.Gui
         private void OnAddToNewPlaylist (object o, EventArgs args)
         {
             // TODO generate name based on the track selection, or begin editing it
-            PlaylistSource playlist = new PlaylistSource ("New Playlist", ActivePrimarySource);
+            PlaylistSource playlist = new PlaylistSource (Catalog.GetString ("New Playlist"), ActivePrimarySource);
             playlist.Save ();
             playlist.PrimarySource.AddChildSource (playlist);
             ThreadAssist.SpawnFromMain (delegate {
@@ -398,6 +416,41 @@ namespace Banshee.Gui
             }
         }
 
+        private void OnOpenContainingFolder (object o, EventArgs args)
+        {
+            var source = ActiveSource as ITrackModelSource;
+
+            if (source == null ||
+                source.TrackModel == null ||
+                source.TrackModel.SelectedItems == null ||
+                source.TrackModel.SelectedItems.Count != 1) {
+                Log.Error ("Could not open containing folder");
+                return;
+            }
+
+            foreach (var track in source.TrackModel.SelectedItems) {
+                var path = System.IO.Path.GetDirectoryName (track.Uri.AbsolutePath);
+                if (Banshee.IO.Directory.Exists (path)) {
+                    System.Diagnostics.Process.Start (path);
+                    return;
+                }
+            }
+
+            var md = new HigMessageDialog (
+                ServiceManager.Get<GtkElementsService> ("GtkElementsService").PrimaryWindow,
+                DialogFlags.DestroyWithParent, MessageType.Warning,
+                ButtonsType.None, Catalog.GetString ("The folder could not be found."),
+                Catalog.GetString ("Please check that the track's location is accessible by the system.")
+            );
+            md.AddButton ("gtk-ok", ResponseType.Ok, true);
+
+            try {
+                md.Run ();
+            } finally {
+                md.Destroy ();
+            }
+        }
+
         private void OnDeleteTracksFromDrive (object o, EventArgs args)
         {
             ITrackModelSource source = ActiveSource as ITrackModelSource;
@@ -444,12 +497,12 @@ namespace Banshee.Gui
             if (!source.ConfirmRemoveTracks) {
                 return true;
             }
-            
+
             bool ret = false;
             string header = null;
             string message = null;
             string button_label = null;
-            
+
             if (delete) {
                 header = String.Format (
                     Catalog.GetPluralString (
@@ -469,15 +522,16 @@ namespace Banshee.Gui
                 );
                 button_label = "gtk-remove";
             }
-                
+
             HigMessageDialog md = new HigMessageDialog (
                 ServiceManager.Get<GtkElementsService> ("GtkElementsService").PrimaryWindow,
                 DialogFlags.DestroyWithParent, delete ? MessageType.Warning : MessageType.Question,
                 ButtonsType.None, header, message
             );
-            md.AddButton ("gtk-cancel", ResponseType.No, false);
-            md.AddButton (button_label, ResponseType.Yes, false);
-            
+            // Delete from Disk defaults to Cancel and the others to OK/Confirm.
+            md.AddButton ("gtk-cancel", ResponseType.No, delete);
+            md.AddButton (button_label, ResponseType.Yes, !delete);
+
             try {
                 if (md.Run () == (int) ResponseType.Yes) {
                     ret = true;
@@ -487,5 +541,11 @@ namespace Banshee.Gui
             }
             return ret;
         }
+
+        // Reserve strings in preparation for the 1.5.5 string freeze (BGO#611923)
+        public void ReservedStrings ()
+        {
+            Catalog.GetString ("View Track Information");
+        }
     }
 }
