@@ -35,6 +35,11 @@ namespace Hyena.Data.Gui
 {
     public partial class ListView<T> : ListViewBase
     {
+        #pragma warning disable 0067
+        public event EventHandler ModelChanged;
+        public event EventHandler ModelReloaded;
+        #pragma warning restore 0067
+
         public void SetModel (IListModel<T> model)
         {
             SetModel (model, 0.0);
@@ -50,44 +55,67 @@ namespace Hyena.Data.Gui
                 model.Cleared -= OnModelClearedHandler;
                 model.Reloaded -= OnModelReloadedHandler;
             }
-            
-            model = value;
 
-            ISortable sortable = model as ISortable;
-            if (sortable != null && sortable.SortColumn == null && ColumnController.DefaultSortColumn != null) {
-                sortable.Sort (ColumnController.DefaultSortColumn);
-            }
+            model = value;
 
             if (model != null) {
                 model.Cleared += OnModelClearedHandler;
                 model.Reloaded += OnModelReloadedHandler;
                 selection_proxy.Selection = model.Selection;
+                IsEverReorderable = model.CanReorder;
             }
-            
+
+            ISortable sortable = model as ISortable;
+            if (sortable != null) {
+                ISortableColumn sort_column = ColumnController.SortColumn ?? ColumnController.DefaultSortColumn;
+                if (sort_column != null) {
+                    sortable.Sort (sort_column);
+                    RecalculateColumnSizes ();
+                    RegenerateColumnCache ();
+                    InvalidateHeader ();
+                    IsReorderable = sortable.SortColumn == null || sortable.SortColumn.SortType == SortType.None;
+                }
+            }
+
+            if (ViewLayout != null) {
+                ViewLayout.Model = Model;
+            }
+
             RefreshViewForModel (vpos);
+
+            var handler = ModelChanged;
+            if (handler != null) {
+                handler (this, EventArgs.Empty);
+            }
         }
 
         private void RefreshViewForModel (double? vpos)
         {
+            if (Model == null) {
+                UpdateAdjustments ();
+                QueueDraw ();
+                return;
+            }
+
+            if (ViewLayout != null) {
+                ViewLayout.ModelUpdated ();
+            }
+
             UpdateAdjustments ();
 
             if (vpos != null) {
-                ScrollTo ((double) vpos);
-            } else {
-                if (Model.Count <= RowsInView) {
-                    // If our view fits all rows at once, make sure we're scrolled to the top
-                    ScrollTo (0.0);
-                } else {
-                    if (vadjustment != null) {
-                        ScrollTo (vadjustment.Value);
-                    }
-                }
+                ScrollToY ((double) vpos);
+            } else if (Model.Count <= ItemsInView) {
+                // If our view fits all rows at once, make sure we're scrolled to the top
+                ScrollToY (0.0);
+            } else if (vadjustment != null) {
+                ScrollToY (vadjustment.Value);
             }
-            
+
             if (Model != null && Selection != null) {
                 Selection.MaxIndex = Model.Count - 1;
             }
-            
+
             if (Parent is ScrolledWindow) {
                 Parent.QueueDraw ();
             }
@@ -97,10 +125,15 @@ namespace Hyena.Data.Gui
         {
             OnModelCleared ();
         }
-        
+
         private void OnModelReloadedHandler (object o, EventArgs args)
         {
             OnModelReloaded ();
+
+            var handler = ModelReloaded;
+            if (handler != null) {
+                handler (this, EventArgs.Empty);
+            }
         }
 
         private void OnColumnControllerUpdatedHandler (object o, EventArgs args)
@@ -112,79 +145,79 @@ namespace Hyena.Data.Gui
         {
             RefreshViewForModel (null);
         }
-        
+
         protected virtual void OnModelReloaded ()
         {
             RefreshViewForModel (null);
         }
-        
+
         private IListModel<T> model;
         public virtual IListModel<T> Model {
             get { return model; }
         }
-        
-        private string row_sensitive_property_name = "Sensitive";
-        private PropertyInfo row_sensitive_property_info;
-        bool row_sensitive_property_invalid = false;
-        
-        public string RowSensitivePropertyName {
-            get { return row_sensitive_property_name; }
-            set { 
-                if (value == row_sensitive_property_name) {
+
+        private string row_opaque_property_name = "Sensitive";
+        private PropertyInfo row_opaque_property_info;
+        bool row_opaque_property_invalid = false;
+
+        public string RowOpaquePropertyName {
+            get { return row_opaque_property_name; }
+            set {
+                if (value == row_opaque_property_name) {
                     return;
                 }
-                
-                row_sensitive_property_name = value;
-                row_sensitive_property_info = null;
-                row_sensitive_property_invalid = false;
-                
+
+                row_opaque_property_name = value;
+                row_opaque_property_info = null;
+                row_opaque_property_invalid = false;
+
                 InvalidateList ();
             }
         }
-        
-        private bool IsRowSensitive (object item)
+
+        private bool IsRowOpaque (object item)
         {
-            if (item == null || row_sensitive_property_invalid) {
+            if (item == null || row_opaque_property_invalid) {
                 return true;
             }
-         
-            if (row_sensitive_property_info == null || row_sensitive_property_info.ReflectedType != item.GetType ()) {
-                row_sensitive_property_info = item.GetType ().GetProperty (row_sensitive_property_name);
-                if (row_sensitive_property_info == null || row_sensitive_property_info.PropertyType != typeof (bool)) {
-                    row_sensitive_property_info = null;
-                    row_sensitive_property_invalid = true;
+
+            if (row_opaque_property_info == null || row_opaque_property_info.ReflectedType != item.GetType ()) {
+                row_opaque_property_info = item.GetType ().GetProperty (row_opaque_property_name);
+                if (row_opaque_property_info == null || row_opaque_property_info.PropertyType != typeof (bool)) {
+                    row_opaque_property_info = null;
+                    row_opaque_property_invalid = true;
                     return true;
                 }
             }
-            
-            return (bool)row_sensitive_property_info.GetValue (item, null);
+
+            return (bool)row_opaque_property_info.GetValue (item, null);
         }
-        
+
         private string row_bold_property_name = "IsBold";
         private PropertyInfo row_bold_property_info;
         bool row_bold_property_invalid = false;
-        
+
         public string RowBoldPropertyName {
             get { return row_bold_property_name; }
-            set { 
+            set {
                 if (value == row_bold_property_name) {
                     return;
                 }
-                
+
                 row_bold_property_name = value;
                 row_bold_property_info = null;
                 row_bold_property_invalid = false;
-                
+
                 InvalidateList ();
             }
         }
-        
+
         private bool IsRowBold (object item)
         {
             if (item == null || row_bold_property_invalid) {
                 return false;
             }
-         
+
             if (row_bold_property_info == null || row_bold_property_info.ReflectedType != item.GetType ()) {
                 row_bold_property_info = item.GetType ().GetProperty (row_bold_property_name);
                 if (row_bold_property_info == null || row_bold_property_info.PropertyType != typeof (bool)) {
@@ -193,17 +226,8 @@ namespace Hyena.Data.Gui
                     return false;
                 }
             }
-            
+
             return (bool)row_bold_property_info.GetValue (item, null);
         }
-        
-        #pragma warning disable 0169
-        
-        private bool IsRowSensitive (int index)
-        {
-            return IsRowSensitive (model[index]);
-        }
-        
-        #pragma warning restore 0169
     }
 }
